@@ -4,6 +4,8 @@
 """
 import math
 from typing import Tuple, Union
+from typing import List, Optional, Sequence
+import numpy as np
 
 import pandas as pd
 from bokeh.models import (
@@ -18,6 +20,8 @@ from bokeh.models import (
     Range1d,
     Tabs,
     Title,
+    PrintfTickFormatter,
+    ColumnDataSource,
 )
 
 # pylint: disable=no-name-in-module
@@ -30,6 +34,8 @@ from ..intermediate import Intermediate, ColumnMetadata
 from ..utils import cut_long_name, fuse_missing_perc, relocate_legend
 from .compute import LABELS
 from ..palette import PALETTE
+from ..palette import BIPALETTE
+
 
 # pylint: enable=no-name-in-module
 
@@ -38,7 +44,10 @@ __all__ = ["render_missing"]
 
 
 def render_missing(
-    itmdt: Intermediate, plot_width: int = 500, plot_height: int = 500,
+    itmdt: Intermediate,
+    plot_width: int = 500,
+    plot_height: int = 500,
+    palette: Optional[Sequence[str]] = None,
 ) -> LayoutDOM:
     """
     @Jinglin write here
@@ -49,6 +58,11 @@ def render_missing(
         return render_missing_impact_1vn(itmdt, plot_width, plot_height)
     elif itmdt.visual_type == "missing_impact_1v1":
         return render_missing_impact_1v1(itmdt, plot_width, plot_height)
+    elif itmdt.visual_type == "missing_spectrum_heatmap":
+        return render_missing_heatmap(
+            itmdt, plot_width, plot_height, palette or BIPALETTE
+        )
+
     else:
         raise UnreachableError
 
@@ -240,6 +254,157 @@ def create_color_mapper() -> Tuple[LinearColorMapper, ColorBar]:
     return mapper, colorbar
 
 
+def create_color_mapper_heatmap(
+    palette: Sequence[str],
+) -> Tuple[LinearColorMapper, ColorBar]:
+    """
+    Create a color mapper and a colorbar for heatmap
+    """
+    mapper = LinearColorMapper(palette=palette, low=-1, high=1)
+    colorbar = ColorBar(
+        color_mapper=mapper,
+        major_label_text_font_size="8pt",
+        ticker=BasicTicker(),
+        formatter=PrintfTickFormatter(format="%.2f"),
+        label_standoff=6,
+        border_line_color=None,
+        location=(0, 0),
+    )
+    return mapper, colorbar
+
+
+def render_missing_heatmap(
+    itmdt: Intermediate, plot_width: int, plot_height: int, palette: Sequence[str]
+) -> Tabs:
+    """
+    Render correlation heatmaps in to tabs
+    """
+    tabs: List[Panel] = []
+    fig_barchart = render_bar_viz(itmdt, "linear", plot_width, plot_height, False)
+    pan_barchart = Panel(child=fig_barchart, title="Barchart")
+    tabs.append(pan_barchart)
+    fig_spectrum = render_missing_spectrum(itmdt, plot_width, plot_height)
+    pan_spectrum = Panel(child=fig_spectrum, title="Spectrum")
+    tabs.append(pan_spectrum)
+    fig_heatmap = render_heatmaps_tab(itmdt, plot_width, plot_height, palette)
+    pan_heatmap = Panel(child=fig_heatmap, title="Heatmap")
+    tabs.append(pan_heatmap)
+
+    tabs = Tabs(tabs=tabs)
+    return tabs
+
+
+def render_heatmaps_tab(
+    itmdt: Intermediate, plot_width: int, plot_height: int, palette: Sequence[str]
+) -> Figure:
+    """
+    Render missing heatmaps in to tabs
+    """
+    tooltips = [("x", "@x"), ("y", "@y"), ("correlation", "@correlation{1.11}")]
+    axis_range = itmdt["axis_range"]
+    df = itmdt["data_heatmap"]
+    df = df.where(
+        np.triu(np.ones(df.shape)).astype(np.bool)  # pylint: disable=no-member
+    ).T
+    df = df.unstack().reset_index(name="correlation")
+    df = df.rename(columns={"level_0": "x", "level_1": "y"})
+    df = df[df["x"] != df["y"]]
+    df = df.dropna()
+    # in case of numerical column names
+    df["x"] = df["x"].apply(str)
+    df["y"] = df["y"].apply(str)
+    mapper, color_bar = create_color_mapper_heatmap(palette)
+    x_range = FactorRange(*axis_range)
+    y_range = FactorRange(*reversed(axis_range))
+    fig = Figure(
+        x_range=x_range,
+        y_range=y_range,
+        plot_width=plot_width,
+        plot_height=plot_height,
+        x_axis_location="below",
+        tools="hover",
+        toolbar_location=None,
+        tooltips=tooltips,
+        background_fill_color="#fafafa",
+    )
+
+    tweak_figure(fig)
+    fig.grid.grid_line_color = None
+    fig.axis.axis_line_color = None
+    fig.rect(
+        x="x",
+        y="y",
+        width=1,
+        height=1,
+        source=df,
+        fill_color={"field": "correlation", "transform": mapper},
+        line_color=None,
+    )
+
+    fig.add_layout(color_bar, "right")
+
+    return fig
+
+
+def render_bar_viz(
+    itmdt: Intermediate,
+    yscale: str,
+    plot_width: int,
+    plot_height: int,
+    show_yticks: bool,
+) -> Figure:
+    """
+    Render a bar chart
+    """
+    # pylint: disable=too-many-arguments
+    length = itmdt["len_data"]
+    df = itmdt["data_barchart"]
+    df = df.copy()
+    df = df.reset_index()
+    df.columns = ["col", "values"]
+    df["val_per"] = df["values"] * length
+    df["missing"] = length - df["val_per"]
+    stack_present = df["val_per"].tolist()
+    stack_missing = df["missing"].tolist()
+    source = ColumnDataSource(df)
+    features = source.data["col"].tolist()
+
+    colors = [PALETTE[0], PALETTE[2]]
+    value_type = ["present", "missing"]
+    data = {"features": features, "present": stack_present, "missing": stack_missing}
+
+    if show_yticks:
+        if len(df) > 10:
+            plot_width = 28 * len(df)
+
+    fig = Figure(
+        x_range=features,
+        plot_width=plot_width,
+        plot_height=plot_height,
+        y_axis_type=yscale,
+        toolbar_location=None,
+        tooltips="$name @features: @$name",
+        tools="hover",
+    )
+
+    fig.vbar_stack(
+        value_type,
+        x="features",
+        width=0.9,
+        color=colors,
+        source=data,
+        legend_label=value_type,
+    )
+
+    fig.legend.location = "top_right"
+    fig.y_range.start = 0
+    fig.x_range.range_padding = 0.1
+    tweak_figure(fig)
+    fig.yaxis.axis_label = "Total Count"
+    relocate_legend(fig, "right")
+    return fig
+
+
 def render_missing_spectrum(
     itmdt: Intermediate, plot_width: int, plot_height: int
 ) -> Figure:
@@ -247,9 +412,7 @@ def render_missing_spectrum(
     Render the missing specturm
     """
     mapper, color_bar = create_color_mapper()
-
     df = itmdt["data"].copy()
-
     df["column_with_perc"] = df["column"].apply(
         lambda c: fuse_missing_perc(cut_long_name(c), itmdt["missing_percent"][c])
     )
@@ -277,7 +440,7 @@ def render_missing_spectrum(
             y_range=y_range,
             plot_width=plot_width,
             plot_height=plot_height,
-            x_axis_location="above",
+            x_axis_location="below",
             tools="hover",
             toolbar_location=None,
             tooltips=tooltips,
@@ -296,7 +459,6 @@ def render_missing_spectrum(
         fill_color={"field": "missing_rate", "transform": mapper},
         line_color=None,
     )
-
     fig.add_layout(color_bar, "right")
 
     return fig
